# Copyright (c) 2014, University of Bristol and University of Southampton.

from EStat.Templating import Template

class HDI(Template):
    'Takes all vectors from current dataset with a name prefixed by stub user defines, then for each calculates highest density interval from a sample of representative values, estimated as shortest credible interval. Modified from Kruschke (2010).'

    __version__ = '1.0.2'

    tags = [ 'Plots' ]
    engines = ['Python_script']

    inputs = '''
stub = Text('Chain name stub: ')
credMass = Text('Interval: ', help='Enter a value between 0 and 1, indicating the mass within the credible interval that is to be estimated.')
'''

    pythonscript = '''
from io import BytesIO

import numpy

import EStat
from EStat.Templating import *

matchvars = filter(lambda x: x.startswith(str(stub)), datafile.variables.keys())
mydata = numpy.ma.column_stack([datafile.variables[i]['data'] for i in matchvars])

# Creates table, adds headings, creates row (list) to which following stats appended
tabout = TabularOutput()
tabout.column_headings = ['Parameter', 'HDI Min', 'Mean', 'Median', 'HDI Max', 'propn>0']

for i in range(0, mydata.shape[1]):

    # Creates sortedPts by sorting relevant col into ascending order
    sortedPts = numpy.sort(mydata[:,i])
    print(sortedPts)
    # ciIdxInc is the number of values (rounded down to nearest integer) to be included within the credible interval
    ciIdxInc = int(float(credMass) * len(sortedPts))

    # Creates nCIs by taking ciIdxInc away from length of sortedPts
    # nCIs is the number of values to be excluded from the credible interval
    nCIs = len(sortedPts) - ciIdxInc

    # For each possible credible interval, calculates difference between values, and returns them in ciWidth
    ciWidth = []
    for j in range(0, nCIs):
        ciWidth.append(sortedPts[j + ciIdxInc] - sortedPts[j])

    row = []
 
    # Creates HDImin: the value in the same location as the minimum value in ciWidth
    HDImin = sortedPts[numpy.argmin(ciWidth)]
    row.append(HDImin)

    # Mean of original chain
    HDImean = numpy.mean(mydata[:,i])
    row.append(HDImean)

    # Median of original chain
    HDImed = numpy.median(mydata[:,i])
    row.append(HDImed)

    # Creates HDImax: the value in the same location as the minimum value in ciWidth,
    # having first added the number of values contained within the interval
    HDImax = sortedPts[numpy.argmin(ciWidth) + ciIdxInc]
    row.append(HDImax)

    PropOverZero = float(sum((mydata[:,i])>0))/len((mydata[:,i]))
    row.append(PropOverZero)

    # Append name of parameter to start of row
    tabout.add_row(matchvars[i], row)
# Finishes assembling table, and declared as an output

outputs['table'] = tabout

'''
